using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using SqlFixture;
using Example1Library;

namespace Example1NUnitTests
{
	[TestFixture]
	public class MultiplicationTableTests
	{
		protected TestDatabase testDatabase;
		protected TestInformation testInfo;

		/// <summary>
		/// Load our test information and set up our test database when we initialize.
		/// </summary>
		public MultiplicationTableTests()
		{
			// Load test information using the key(s) defined in app.config's TestInformation section
			testInfo = new TestInformation("MyTestInformation");
			
			// Load database information using the key(s) defined in app.config's TestDatabase section
			testDatabase = new TestDatabase("MyTestDatabase");
		}

		/// <summary>
		/// Set up each test by replacing the database with the values in our SQL file.
		/// </summary>
		[SetUp]
		public void SetUp()
		{
			// Run our SQL file on the test database using the key(s) defined in app.config's SqlFiles section.
			// There are actually multiple ways to execute SQL scripts on a test database using the TestDatabase object.
			testDatabase.LoadSqlFixture("MySqlFile");
		}

		/// <summary>
		/// Test that the value of 3 times 4 is 12.
		/// </summary>
		[Test]
		public void PositiveTest()
		{
			// Pull our initial values and expected data from the test information XML file
			// We can call GetSettingValue and GetExpectedValue with the fixture and test explicitly named,
			// but if you have named your Fixture and Test elements the same as the fixture class and test
			// method, SqlFixture figures this out through the automagic of reflection.
			int number1 = testInfo.GetSettingValue<int>("Number1");
			int number2 = testInfo.GetSettingValue<int>("Number2");
			
			// We can actually name multiple expected values, but if no name is given to the Expected element,
			IExpected expectedObject = testInfo.GetExpectedValue();
			int expected = expectedObject.GetValue<int>();

			int actual = MultiplicationTables.GetValue(number1, number2);
			Assert.IsTrue(expectedObject.IsExpected(actual));
			Assert.AreEqual(expected, actual);
		}

		/// <summary>
		/// Test two values that do not exist in the database. This should throw an exception.
		/// </summary>
		[Test]
		[ExpectedException(typeof(NotFoundException))]
		public void NegativeTest()
		{
			// Pull our initial values and expected data from the test information XML file
			int number1 = testInfo.GetSettingValue<int>("Number1");
			int number2 = testInfo.GetSettingValue<int>("Number2");

			// Here's where we should throw the exception
			int actual = MultiplicationTables.GetValue(number1, number2);
		}

		/// <summary>
		/// Test that the value of 3 times 4 is 13. Our database is correct in this case, so this test will always fail.
		/// </summary>
		[Test]
		public void IntentionallyFailedTest()
		{
			// Pull our initial values and expected data from the test information XML file
			int number1 = testInfo.GetSettingValue<int>("Number1");
			int number2 = testInfo.GetSettingValue<int>("Number2");
			IExpected expectedObject = testInfo.GetExpectedValue();
			int expected = expectedObject.GetValue<int>();

			int actual = MultiplicationTables.GetValue(number1, number2);
			Assert.IsTrue(expectedObject.IsExpected(actual));
			Assert.AreEqual(expected, actual);
		}

	}
}
