using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.IO;
using System.Text;

namespace SqlFixture
{
	public class TestDatabase
	{
		string _key;
		string _connectionStringName;

		#region Properties
		public string ConnectionStringName
		{
			get { return _connectionStringName; }
		}

		public string ConnectionString
		{
			get { return ConfigurationManager.ConnectionStrings[_connectionStringName].ConnectionString; }
		}
		#endregion

		#region Static Methods & Properties
		public static NameValueCollection Configuration
		{
			get
			{
				NameValueCollection col = ConfigurationManager.GetSection("SqlFixture/TestDatabases") as NameValueCollection;
				if (col == null) { throw new ConfigurationErrorsException("SqlFixture / TestDatabases configuration section not found."); }
				return col;
			}
		}
		#endregion

		#region Constructors & Initialization
		public TestDatabase(string key)
		{
			_key = key;
			_connectionStringName = Configuration[key];
		}
		#endregion

		#region Execute Queries
		public DataTable ExecuteQuery(string sql)
		{
			SqlConnection conn = new SqlConnection(ConnectionString);
			conn.Open();
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = sql;
			DataTable dt = new DataTable();
			dt.Load(cmd.ExecuteReader());
			conn.Close();
			return dt;
		}

		public int ExecuteNonQuery(string sql)
		{
			SqlConnection conn = new SqlConnection(ConnectionString);
			conn.Open();
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = sql;
			int results = cmd.ExecuteNonQuery();
			conn.Close();
			return results;
		}

		public object ExecuteScalar(string sql)
		{
			SqlConnection conn = new SqlConnection(ConnectionString);
			conn.Open();
			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = sql;
			object results = cmd.ExecuteScalar();
			conn.Close();
			return results;
		}
		#endregion

		#region Common Commands
		public void ClearDatabaseTable(string table)
		{
			ExecuteNonQuery(String.Format("DELETE FROM {0}", table));
		}

		public void ResetIdentityColumn(string table)
		{
			ExecuteNonQuery(String.Format("dbcc checkident({0}, reseed, 0)", table));
		}
		#endregion

		#region Load SQL Scripts
		public void LoadSqlFixture(string key)
		{
			SqlFile sf = new SqlFile(key);
			LoadSqlScript(sf.Contents);
		}

		public void LoadSqlFixtures()
		{
			foreach (string key in SqlFile.Configuration.Keys)
			{
				SqlFile sf = new SqlFile(key);
				LoadSqlScript(sf.Contents);
			}
		}

		public void LoadSqlScript(string sql)
		{
			List<string> buffer = new List<string>();
			string[] lines = sql.Split('\n');
			string cmdText = "";

			foreach (string line in lines)
			{
				if (line.Trim().ToLower() == "go")
				{
					cmdText = String.Join("\n", buffer.ToArray());
					if (!String.IsNullOrEmpty(cmdText.Trim())) { ExecuteNonQuery(cmdText); }
					buffer = new List<string>();
				}
				else
				{
					buffer.Add(line);
				}
			}
			if (buffer.Count > 0)
			{
				cmdText = String.Join("\n", buffer.ToArray());
				if (!String.IsNullOrEmpty(cmdText.Trim())) { ExecuteNonQuery(cmdText); }
			}
		}

		public void LoadSqlFile(string filename)
		{
			FileStream fs = new FileStream(filename, FileMode.Open, FileAccess.Read);
			StreamReader sr = new StreamReader(fs);
			LoadSqlScript(sr.ReadToEnd());
			sr.Close();
		}

		public static void LoadSqlFixture(string dbKey, string scriptKey)
		{
			TestDatabase td = new TestDatabase(dbKey);
			td.LoadSqlFixture(scriptKey);
		}
		#endregion
	}
}
